"use strict";
/* 
  BEGIN LICENSE BLOCK

	QuickFolders is released under the Creative Commons (CC BY-ND 4.0)
	Attribution-NoDerivatives 4.0 International (CC BY-ND 4.0) 
	For details, please refer to license.txt in the root folder of this extension

  END LICENSE BLOCK 
*/

QuickFolders.Preferences = {
	service: Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch),

	get isDebug() {
		return this.getBoolPref("debug");
	},
	
	set lastActiveCats(c) {
		this.setStringPref('lastActiveCategories', c);
	},
	get lastActiveCats() {
		return this.getStringPref('lastActiveCategories');
	},

	isDebugOption: function(option) { // granular debugging
		if(!this.isDebug) return false;
		try {return this.getBoolPref("debug." + option);}
		catch(e) { 
      return true; // more info is probably better in this case - this is an illegal value after all.
    }
	},
	
	setDebugOption: function setDebugOption(option, val) {
		this.setBoolPref("debug." + option, val);
	},

	storeFolderEntries: function storeFolderEntries(folderEntries) {
		try {
			const util = QuickFolders.Util,
			      PS = this.service;
			let json = JSON.stringify(folderEntries),
					str = Components.classes["@mozilla.org/supports-string;1"]
				.createInstance(Components.interfaces.nsISupportsString);
			str.data = json;

			if (PS.setStringPref)
				PS.setStringPref("QuickFolders.folders", json);
			else
				PS.setComplexValue("QuickFolders.folders", Components.interfaces.nsISupportsString, str);
		}
		catch(e) {
			QuickFolders.Util.logToConsole("storeFolderEntries()" + e);
		}
	} ,

	loadFolderEntries: function loadFolderEntries() {
		const setting = "QuickFolders.folders",
		      util = QuickFolders.Util;
		if (!this.service.prefHasUserValue(setting)) {
			return [];
		}

		try {
			const PS = this.service;
			let folders = 
			  (typeof PS.getStringPref === 'function') ?
				PS.getStringPref(setting) :
			  PS.getComplexValue(setting, Components.interfaces.nsISupportsString).data;
			// fall back for old version
			if (folders.length<3)
				folders = PS.getCharPref("QuickFolders.folders");

			if(folders) {
				folders = folders.replace(/\r?\n|\r/, ''); // remove all line breaks
				let entries = JSON.parse(folders);
				for (let i = 0; i < entries.length; i++) {
					if (typeof entries[i].tabColor ==='undefined' || entries[i].tabColor ==='undefined')
						entries[i].tabColor = 0;
					// default the name!!
					if (!entries[i].name) {
						// retrieve the name from the folder uri (prettyName)
						let f = QuickFolders.Model.getMsgFolderFromUri(entries[i].uri, false);
						if (f)
							entries[i].name = f.prettyName;
					}
				}
				return entries;
			}
			else
				return [];
		}
		catch(e) {
			QuickFolders.Util.logToConsole("loadFolderEntries()" + e);
			return [];
		}
	} ,

	get isShowUnreadCount() {
		return this.getBoolPref("showUnreadOnButtons");
	} ,

	get isShowQuickFoldersLabel() {
		return this.getBoolPref("showQuickfoldersLabel");
	} ,

	get isShowUnreadFoldersBold() {
		return this.getBoolPref("showUnreadFoldersBold");
	} ,

	get isHighlightNewMail() {
		return this.getBoolPref("showNewMailHighlight");
	} ,
	
	get isItalicsNewMail() { // xxx experimental
		return this.getBoolPref("showFoldersWithNewMailItalic");
	} ,

	get isShowRecursiveFolders() {
		return this.getBoolPref("showSubfolders");
	} ,

	get isShowCategoryNewCount() {
		return this.getBoolPref("showCategoryCounts");
	} ,
	
	get isKeyboardListeners() {
		return this.isUseNavigateShortcuts 
		    || this.isUseKeyboardShortcuts 
				|| this.isUseRebuildShortcut 
		    || this.isQuickJumpShortcut 
				|| this.isQuickMoveShortcut 
				|| this.isQuickCopyShortcut
				|| this.isSkipFolderShortcut;
	} ,

	get isUseNavigateShortcuts() {
		return this.getBoolPref("useNavigateShortcuts");
	} ,

	get isUseKeyboardShortcuts() {
		return this.getBoolPref("useKeyboardShortcuts");
	} ,

	get isUseRebuildShortcut() {
		return this.getBoolPref("useRebuildShortcut");
	} ,

	get RebuildShortcutKey() {
		return this.getStringPref("rebuildShortcutKey");
	} ,
	
	get isQuickJumpShortcut() {
		return this.getBoolPref("quickJump.useHotkey");
	} ,
	
	get QuickJumpShortcutKey() {
		return this.getStringPref("quickJump.Hotkey");
	} ,

	get isQuickMoveShortcut() {
		return this.getBoolPref("quickMove.useHotkey");
	} ,
	
	get QuickMoveShortcutKey() {
		return this.getStringPref("quickMove.Hotkey");
	} ,
  
	get isQuickCopyShortcut() {
		return this.getBoolPref("quickCopy.useHotkey");
	} ,
  
	get QuickCopyShortcutKey() {
		return this.getStringPref("quickCopy.Hotkey");
	} ,
	
	get isSkipFolderShortcut() {
		return this.getBoolPref("skipFolder.useHotkey");
	} ,
  
	get SkipFolderShortcutKey() {
		return this.getStringPref("skipFolder.Hotkey");
	} ,
	

	get isUseKeyboardShortcutsCTRL() {
		return this.getBoolPref("useKeyboardShortcutCTRL");
	} ,

	get isShowShortcutNumbers() {
		return this.getBoolPref("showShortcutNumber");
	} ,

	get isShowTotalCount() {
		return this.getBoolPref("showTotalNumber");
	} ,

	get isShowCountInSubFolders() {
		return this.getBoolPref("showCountInSubFolders");
	} ,

	get isShowFoldersWithMessagesItalic() {
		return this.getBoolPref("showFoldersWithMessagesItalic");
	} ,

	get isFocusPreview() {
		return this.getBoolPref("autoFocusPreview");
	} ,

	get isShowToolbarIcons() {
		return this.getBoolPref("showIcons");
	} ,

	get isChangeFolderTreeViewEnabled() {
		return !this.getBoolPref('disableFolderSwitching');
	} ,

	get isSortSubfolderMenus() {
		return this.getBoolPref('enableMenuAlphaSorting');
	} ,

	get isShowRecentTab() {
		return this.getBoolPref('showRecentTab');
	} ,

	get isShowRecentTabIcon() {
		return this.getBoolPref('recentfolders.showIcon');
	} ,

	get isPastelColors() { // OBSOLETE!
		return this.getBoolPref('pastelColors');
	} ,

	get recentTabColor() {
		return this.getIntPref( 'recentfolders.color');
	} ,


	get isMinimalUpdateDisabled() {
		return this.getBoolPref('update.disableMinimal');
	} ,

	get isShowToolIcon() {
		return this.getBoolPref('showToolIcon');
	} ,
  
  get isShowReadingList() {
		return this.getBoolPref('bookmarks.showButton');
  } ,
  
  get isShowQuickMove() {
    return this.getBoolPref('showQuickMove');
  } ,
	get isCssTransitions() {
		return this.getBoolPref('style.transitions');
	} ,

	get ButtonFontSize() {
		return this.getIntPref("buttonFontSizeN");
	} ,

	get TextQuickfoldersLabel() {
		const licenser = QuickFolders.Licenser,
		      util = QuickFolders.Util;
		let renewalLabel = (licenser.isExpired) 
		     ? util.getBundleString("qf.notification.premium.btn.renewLicense", "Renew License!")
				 : "";
		try { // to support UNICODE: https://developer.mozilla.org/pl/Fragmenty_kodu/Preferencje
		  const url = "extensions.quickfolders.textQuickfoldersLabel",
					  PS = this.service;
			let customTitle = 
			  (typeof PS.getStringPref === 'function') ?
				this.service.getStringPref(url) :
			  this.service.getComplexValue(url, Components.interfaces.nsISupportsString).data;
			return renewalLabel || customTitle;
		}
		catch(e) { return renewalLabel || 'QuickFolders'; }
	},

  get maxSubjectLength() {
    return this.getIntPref('menuMessageList.maxSubjectLength');
  } ,
  
	get ColoredTabStyle() {
		// 0 - filled
		// 1 - striped
		return this.getIntPref('colorTabStyle');
	} ,

	TABS_STRIPED: 0,
	TABS_FILLED: 1,

	existsCharPref: function existsCharPref(pref) {
		try {
			if(this.service.prefHasUserValue(pref))
				return true;
			if (this.service.getCharPref(pref))
				return true;
		}
		catch (e) {return false; }
		return false;
	},

	existsBoolPref: function existsBoolPref(pref) {
		try {
			if(this.service.prefHasUserValue(pref))
				return true;
			if (this.service.getBoolPref(pref))
				return true;
		}
		catch (e) {return false; }
		return false;
	},

	// updates all toxic preferences to skinning engine of v 2.7
	// returns true if upgraded from a previous skinning engine
	tidyUpBadPreferences: function tidyUpBadPreferences() {
		let isUpgradeSkinning = false,
        util = QuickFolders.Util;
		try {
			util.logDebugOptional('firstrun', 'tidyUpBadPreferences() ...');
			// get rid of preferences that do not start with "preferences." and replace with newer versions.
			if (this.existsCharPref('QuickFolders.Toolbar.Style.background-color')) {
				let replacePref = function(id1, id2) {
					let service = QuickFolders.Preferences.service,
					    origPref = 'QuickFolders.' + id1 + '.Style.' + id2,
					    sValue = "";
					// save value set by user (if none, default will create it)
					if (service.prefHasUserValue(origPref)) {
						sValue = service.getCharPref(origPref);
						let newPref = 'extensions.quickfolders.style.' + id1 + '.' + id2;
						service.setCharPref(newPref, sValue);
						util.logDebugOptional('firstrun', 'QuickFolders Update: Replaced bad preference {' + origPref + '} with {' + newPref + '}  value=' + sValue );
					}
					// delete bad preference
					try { service.deleteBranch(origPref) } catch (ex) {;};
				}

				replacePref('ActiveTab','background-color');
				replacePref('ActiveTab','color');
				replacePref('DragOver','background-color');
				replacePref('DragTab','background-color');
				replacePref('DragTab','color');
				replacePref('HoveredTab','background-color');
				replacePref('HoveredTab','color');
				replacePref('InactiveTab','background-color');
				replacePref('InactiveTab','color');
				replacePref('Toolbar','background-color');
				isUpgradeSkinning = true;

			}

			if (this.existsBoolPref("extensions.quickfolders.showFlatStyle")) {
				let theme = QuickFolders.Themes;

				if (this.getBoolPref("showFlatStyle"))
					this.CurrentThemeId = theme.themes.Flat.Id;
				else {
					if (this.getBoolPref("showNativeTabStyle"))
						this.CurrentThemeId = theme.themes.NativeTabs.Id;
					else
						this.CurrentThemeId = theme.themes.ApplePills.Id; // Pills style
				}

				try {
					this.service.deleteBranch("extensions.quickfolders.showFlatStyle");
					this.service.deleteBranch("extensions.quickfolders.showNativeTabStyle");
					isUpgradeSkinning = true;
				}
				catch (ex) { util.alert(ex);};
			}
		}
		catch (ex) {
			util.logException("tidyUpBadPreferences",ex) ;
			return false;
		};
		return isUpgradeSkinning;
	},

	getUserStyle: function getUserStyle(sId, sType, sDefault) {
		// note: storing color as string in order to store OS specific colors like Menu, Highlight
		// usage: getUserStyle("ActiveTab","background-color","HighLight")
		// usage: getUserStyle("ActiveTab","color", "HighlightText")
		let sStyleName = 'extensions.quickfolders.style.' + sId + '.' + sType,
		    sReturnValue="";

    try {
			let localPref = 
        (typeof sDefault == "string") ?
        this.service.getCharPref(sStyleName) :
        this.service.getIntPref(sStyleName);
			if (localPref || (localPref===0))
				sReturnValue = localPref;
			else
				sReturnValue = sDefault;
		}
    catch(ex) {
      sReturnValue = sDefault;
    }
		/* QuickFolders.Util.logToConsole("getUserStyle("+ sId+ ", " + sType + ", " + sDefault +")\n" +
				"Style Name: " + sStyleName + "\n" +
				"Value: " + sReturnValue) */
		return sReturnValue;
	},

	setUserStyle: function setUserStyle(sId, sType, sValue) {
		let sStyleName = 'extensions.quickfolders.style.' + sId + '.' + sType;
		this.service.setCharPref(sStyleName, sValue);
	},

	getIntPreference: function getIntPreference(p) {
		try {
			return this.service.getIntPref(p);
		}
		catch (ex) {
			QuickFolders.Util.logException("getIntPref(" + p + ") failed\n", ex);
			return 0;
		}
	},

	setIntPreference: function setIntPreference(p, v) {
		return this.service.setIntPref(p, v);
	},

	getBoolPrefSilent: function getBoolPrefSilent(p) {
		try {
			return this.service.getBoolPref(p);
		}
		catch(e) {
			return false;
		}
	},

	getBoolPrefVerbose: function getBoolPrefVerbose(p) {
		try {
			return this.service.getBoolPref(p);
		}
		catch(e) {
			QuickFolders.Util.logException("getBoolPrefVerbose(" + p + ") failed\n", e);
			return false;
		}
	},
	
	getBoolPref: function getBoolPref(p) {
	  let ans;
	  try {
	    ans = this.service.getBoolPref("extensions.quickfolders." + p);
		}
		catch(ex) {
		  QuickFolders.Util.logException("getBoolPref("  + p +") failed\n", ex);
		  throw(ex);
		}
		return ans;
	},

	setBoolPref: function setBoolPref(p, v) {
		return this.service.setBoolPref("extensions.quickfolders." + p, v);
	},
	
	getFiltersBoolPref: function getFiltersBoolPref(p, defaultV) {
	  let ans;
	  try {
	    ans = this.service.getBoolPref("extensions.quickfilters." + p);
		}
		catch(ex) {
		  QuickFolders.Util.logException("getFiltersBoolPref("  + p +") failed\ndefaulting to " + defaultV, ex);
			ans = defaultV;
		}
		return ans;
	},
	
	getStringPref: function getStringPref(p) {
    let prefString ='',
		    key = "extensions.quickfolders." + p;        
		  
    try {
		  const Ci = Components.interfaces, Cc = Components.classes;
			prefString = 
				this.service.getStringPref ?
				this.service.getStringPref(key) :
				this.service.getComplexValue(key, Ci.nsISupportsString).data;			
    }
    catch(ex) {
      QuickFolders.Util.logDebug("Could not retrieve string pref: " + p + "\n" + ex.message);
    }
    finally {
      return prefString;
    }
	},
	
	setStringPref: function setStringPref(p, v) {
		if (this.service.setStringPref)
			return this.service.setStringPref("extensions.quickfolders." + p, v);
		else {
		  const Ci = Components.interfaces, 
			      Cc = Components.classes;
			if (this.isDebug) debugger;
			var str = Cc["@mozilla.org/supports-string;1"].createInstance(Ci.nsISupportsString);
			str.data = v;
			Services.prefs.setComplexValue("extensions.quickfolders." + p, Ci.nsISupportsString, str);
		}
	},

	getIntPref: function getIntPref(p) {
		return QuickFolders.Preferences.getIntPreference("extensions.quickfolders." + p);
	},

	setIntPref: function setIntPref(p, v) {
		return this.setIntPreference("extensions.quickfolders." + p, v);
	},

	setShowCurrentFolderToolbar: function setShowCurrentFolderToolbar(b, selector) {
		let tag = "showCurrentFolderToolbar";
		if (selector)
			tag = tag + "." + selector;
		return this.service.setBoolPref("extensions.quickfolders." + tag, b);
	},

	isShowCurrentFolderToolbar: function isShowCurrentFolderToolbar(selector) {
		let tag = "showCurrentFolderToolbar";
		if (selector)
			tag = tag + "." + selector;
		return QuickFolders.Preferences.getBoolPref(tag, false);
	},

	setBoolPrefVerbose: function setBoolPrefVerbose(p, v) {
		try {
			return this.service.setBoolPref(p, v);
		} 
    catch(e) {
			let s="Err:" +e;
			QuickFolders.Util.logException("setBoolPrefVerbose(" + p + ") failed\n", e);
			return false;
		}
	} ,

	get CurrentTheme() {
		let id = this.CurrentThemeId;
		return QuickFolders.Themes.Theme(id);
	},

	get CurrentThemeId() {
		return this.getStringPref("style.theme");
	},

	set CurrentThemeId(t) {
		return this.setStringPref("style.theme",t);
	},
  
  get supportsCustomIcon() {
    switch(QuickFolders.Util.Application) {
      case "Thunderbird":
        return true;
      default:
        return false; // SeaMonkey and Postbox - custom icons not supported!
    }
  },
	
	unhideSmallIcons() {
		// option: make "small icons" option visible again in customize toolbar palette
		if (this.getBoolPref("toolbarpalette.showSmallIcons")) {
			let option = document.getElementById('smallicons');
			if (option)
				option.style.setProperty('display','block'); // by default this has been set to 'none' for ages
		}
	}
  
}
